@@ -11,7 +11,7 @@ from ..mbta_object_store import MBTARouteObjStore, MBTAStopObjStore
 from ..client.mbta_client import MBTAClient
 
 from ..trip import Trip
-from ..stop import StopType
+from ..stop import Stop, StopType
 
 from ..models.mbta_stop import MBTAStop
 from ..models.mbta_schedule import MBTASchedule
@@ -20,9 +20,9 @@ from ..models.mbta_alert import MBTAAlert, MBTAAlertPassengerActivity, MBTAAlert
 
 class MBTABaseHandler:
 
-    FILTER_TIME_DEPARTURE_BUFFER = timedelta(seconds=90)
-    FILTER_TIME_ARRIVAL_BUFFER = timedelta(seconds=60)
     DEFAULT_MAX_TRIPS = 1
+    
+    FILTER_GRACE_PERIOD = 20 # seconds
 
     def __init__(
         self,
@@ -207,7 +207,6 @@ class MBTABaseHandler:
                         trips.pop(scheduling.trip_id)
                     # skip this scheduling
                     continue
-                
                 # If the trip of the prediction is not in the trips dict
                 if scheduling.trip_id not in trips:
                     # Create the trip
@@ -215,8 +214,8 @@ class MBTABaseHandler:
                 else:
                     # Or get the trip with same trop_id from the trips
                     trip = trips[scheduling.trip_id]
-                    
-                #Set the mbta_route for the trip, 
+
+                #Set the mbta_route for the trip,
                 if not trip.mbta_route and scheduling.route_id:
                     # if the route in the objStore
                     if MBTARouteObjStore.get_by_id(scheduling.route_id):
@@ -227,13 +226,13 @@ class MBTABaseHandler:
                         mbta_route, _ = await self._mbta_client.fetch_route(scheduling.route_id)
                         # set the route for the trip (it will set the route id and store the obj in the objstore)
                         trip.mbta_route = mbta_route
-                
+
                 # if route type 0 or 1 and MBTASchedule skipp
                 #https://www.mbta.com/developers/v3-api/best-practices#predictions
                 if trip.mbta_route.type in [0,1] and isinstance(scheduling, MBTASchedule):
                     #Unfortunately in some corner cases there may be more trips type for the same trip...cannot break
                     continue
-                    
+
                 # Check if the prediction stop_id is in the departure stops lists
                 if scheduling.stop_id in self._mbta_stops_ids[StopType.DEPARTURE]:
                     #add/update the stop as departure
@@ -248,11 +247,11 @@ class MBTABaseHandler:
                         stop_type=StopType.ARRIVAL,
                         scheduling=scheduling,
                         mbta_stop_id=scheduling.stop_id)
-                    
+
                 trips[scheduling.trip_id] = trip
-                    
+
             return trips
-        
+
         except Exception as e:
             self._logger.error(f"Error while processing scheduling: {e}")
             self._logger.error(traceback.format_exc())  # Log the stack trace
@@ -262,20 +261,20 @@ class MBTABaseHandler:
         if any(stop_id in stop_ids for stop_ids in self._mbta_stops_ids.values()):
             return MBTAStopObjStore.get_by_id(stop_id)
         return None
-    
-        
+
+
     ## SET TRIP DETAILS
     async def _update_details(self, trips: dict[str, Trip]) -> dict[str, Trip]:
         """Update trip details with MBTA trip, vehicle, and alert information."""
         self._logger.debug("Updating trips details")
-        
+
         try:
             updated_trips: dict[str, Trip] = {}
 
             # Collect all trip IDs to fetch vehicles
             params = {'filter[trip]': ','.join(trips.keys())}
             mbta_stop_ids = self._mbta_stops_ids.get(StopType.DEPARTURE, []) + self._mbta_stops_ids.get(StopType.ARRIVAL, [])
-            
+
             # Fetch vehicles and alerts concurrently
             task_vehicles = asyncio.create_task(self._mbta_client.fetch_vehicles(params))
             task_alerts = asyncio.create_task(self.__get_mbta_alerts(trips=trips, stops_ids=mbta_stop_ids))
@@ -293,19 +292,21 @@ class MBTABaseHandler:
                 if not trip.mbta_trip:
                     mbta_trip, _ = await self._mbta_client.fetch_trip(id=trip_id)
                     trip.mbta_trip = mbta_trip
+    
+                # If the the trip is day1+ do not add vehicle info
+                if trip.departure_time.date() == datetime.now().date():
 
-                # Match vehicle for the trip
-                matching_vehicles = [vehicle for vehicle in mbta_vehicles if vehicle.trip_id == trip_id]
-
-                # Assign the vehicle to the trip if exist
-                trip.mbta_vehicle = matching_vehicles[0] if len(matching_vehicles) > 0 else None
+                    # Match vehicle for the trip
+                    matching_vehicles = [vehicle for vehicle in mbta_vehicles if vehicle.trip_id == trip_id]
+                    # Assign the vehicle to the trip if exist
+                    trip.mbta_vehicle = matching_vehicles[0] if len(matching_vehicles) > 0 else None
 
                 # Filter and sort relevant alerts
                 processed_alerts = []
                 for mbta_alert in mbta_alerts:
                     if mbta_alert not in processed_alerts and mbta_alert.severity > 1 and self.__is_alert_relevant(mbta_alert, trip):
                         processed_alerts.append(mbta_alert)
-                        
+
                 # Sort processed_alerts by severity (high to low)
                 processed_alerts.sort(key=lambda mbta_alert: mbta_alert.severity, reverse=True)
 
@@ -323,7 +324,7 @@ class MBTABaseHandler:
 
     async def __get_mbta_alerts(
         self,
-        trips: Optional[dict[str, Trip]] = None, 
+        trips: Optional[dict[str, Trip]] = None,
         stops_ids: Optional[list[str]] =  None)-> Tuple[list[MBTAAlert],str]:
         self._logger.debug("Updating MBTA alerts")
         try:
@@ -342,7 +343,7 @@ class MBTABaseHandler:
         except Exception as e:
             self._logger.error(f"Error updating MBTA alerts: {e}")
             raise
-        
+
     def __is_alert_relevant(self, mbta_alert: MBTAAlert, trip: Trip) -> bool:
         """
         Determines whether an MBTA alert is relevant to a specific trip.
@@ -360,9 +361,9 @@ class MBTABaseHandler:
             - The alert's informed entity is tied to the departure or arrival stops with relevant activities (boarding or exiting).
             - The trip's departure or arrival time falls within the alert's active period.
         """
-        
+
         try:
-            
+
             def check_route_level_alert(informed_entity: MBTAAlertsInformedEntity):
                 """
                 Check if the informed entity is a route-level alert matching the trip's route and direction.
@@ -379,37 +380,50 @@ class MBTABaseHandler:
                     not informed_entity.trip_id and
                     (not informed_entity.direction_id or informed_entity.direction_id == trip.mbta_trip.direction_id)
                 )
-            
+
             trip_id = trip.mbta_trip.id if trip.mbta_trip else None
             departure_stop_id = trip.get_stop_id_by_stop_type(StopType.DEPARTURE)
             arrival_stop_id = trip.get_stop_id_by_stop_type(StopType.ARRIVAL)
 
             for informed_entity in mbta_alert.informed_entities:
-                
+
                 active_period_start = mbta_alert.active_period_start.replace(tzinfo=None)
                 active_period_end = mbta_alert.active_period_end.replace(tzinfo=None)
-                
+
                 try:
                     if (
-                        check_route_level_alert(informed_entity) or
-                        informed_entity.trip_id == trip_id or
-                        (
-                            informed_entity.stop_id == departure_stop_id and 
-                            MBTAAlertPassengerActivity.BOARD in informed_entity.activities
-                        ) or
-                        (
-                            informed_entity.stop_id == arrival_stop_id and 
-                            MBTAAlertPassengerActivity.EXIT in informed_entity.activities
+                        check_route_level_alert(informed_entity)
+                        or informed_entity.trip_id == trip_id
+                        or (
+                            informed_entity.stop_id == departure_stop_id
+                            and MBTAAlertPassengerActivity.BOARD.value in informed_entity.activities
+                        )
+                        or (
+                            informed_entity.stop_id == arrival_stop_id
+                            and MBTAAlertPassengerActivity.EXIT.value in informed_entity.activities
                         )
                     ) and (
                         (
-                            trip.departure_time and 
-                            (active_period_start is None or active_period_start <= trip.departure_time) and 
-                            (active_period_end is None or trip.departure_time <= active_period_end)
-                        ) or (
-                            trip.arrival_time and 
-                            (active_period_start is None or active_period_start <= trip.arrival_time) and 
-                            (active_period_end is None or trip.arrival_time <= active_period_end)
+                            trip.departure_time
+                            and (
+                                active_period_start is None
+                                or active_period_start <= trip.departure_time
+                            )
+                            and (
+                                active_period_end is None
+                                or trip.departure_time <= active_period_end
+                            )
+                        )
+                        or (
+                            trip.arrival_time
+                            and (
+                                active_period_start is None
+                                or active_period_start <= trip.arrival_time
+                            )
+                            and (
+                                active_period_end is None
+                                or trip.arrival_time <= active_period_end
+                            )
                         )
                     ):
                         return True
@@ -432,48 +446,55 @@ class MBTABaseHandler:
         remove_departed: Optional[bool] = True,
         require_both_stops: Optional[bool] = True,
         sort_by: Optional[StopType] = StopType.DEPARTURE) -> dict[str, Trip]:
-       
+
         """Filter and sort trips based on conditions like direction, departure, and arrival times."""
         self._logger.debug("Filtering Trips")
         now = datetime.now().astimezone()
         filtered_trips: dict[str, Trip] = {}
 
         try:
-   
-            trips = self._sort_trips(trips, sort_by)
-            
-            for trip_id, trip in trips.items():
-                departure_stop = trip.get_stop_by_type(StopType.DEPARTURE)
-                arrival_stop = trip.get_stop_by_type(StopType.ARRIVAL)
 
-                # Ensure both departure and arrival stops exist and are in the correct sequence
+            trips = self._sort_trips(trips, sort_by)
+
+            for trip_id, trip in trips.items():
+                departure_stop: Optional[Stop] = trip.get_stop_by_type(StopType.DEPARTURE)
+                arrival_stop: Optional[Stop] = trip.get_stop_by_type(StopType.ARRIVAL)
+
+                # If both stops required ensure both departure and arrival stops exist and are in the correct sequence
                 if require_both_stops and (
-                    not departure_stop or not arrival_stop or departure_stop.stop_sequence > arrival_stop.stop_sequence
+                    not departure_stop
+                    or not arrival_stop
+                    or departure_stop.stop_sequence > arrival_stop.stop_sequence
                 ):
                     continue
                 else:
-                    # if sequence cannot be used, check that a DEPARTURE stop has departure Time obj (is not the last stop of a trip) and ARRIVAL stop has arrival Time obj (is not the first stop of a tirp)
+                    # elsecheck that a DEPARTURE stop has departure Time obj (is not the last stop of a trip) and ARRIVAL stop has arrival Time obj (is not the first stop of a tirp)
                     if departure_stop and not departure_stop.departure:
                         continue
                     if arrival_stop and not arrival_stop.arrival:
                         continue
 
-                # Remove trips that have already departed (+BUFFER time)
-                if remove_departed and departure_stop.departure_time and departure_stop.departure_time < now - self.FILTER_TIME_DEPARTURE_BUFFER:
-                    continue
+                now = datetime.now().astimezone()       
 
-                # Remove trips that have already arrived (+BUFFER time)
-                if arrival_stop and arrival_stop.arrival_time and arrival_stop.arrival_time < now - self.FILTER_TIME_ARRIVAL_BUFFER:
-                    continue
-
-                # If vehicle data is available, use it for filtering
-                if trip.mbta_vehicle:
-
-                    if remove_departed and departure_stop and trip.has_departed:
-                        continue  # If the vehicle has passed the departure stop, filter it out
-
-                    if arrival_stop and trip.has_arrived:
-                        continue  # If the vehicle has passed the arrival stop, filter it out
+                # Remove trips that have already departed + REMOVAL_BUFFER_THRESHOLD
+                if remove_departed and departure_stop:
+                    
+                    departure_time = departure_stop.departure_time or departure_stop.time
+                    departure_delta = departure_time.astimezone() - now
+                    seconds_to_departure = int(departure_delta.total_seconds())
+                    
+                    if trip.has_departed(departure_stop, seconds_to_departure=seconds_to_departure, filtering_grace_period=self.FILTER_GRACE_PERIOD):
+                        continue
+                
+                # Remove trips that have already arrived + REMOVAL_BUFFER_THRESHOLD
+                if arrival_stop:
+                   
+                    arrival_time = arrival_stop.arrival_time or arrival_stop.time
+                    arrival_delta = arrival_time.astimezone() - now
+                    seconds_to_arrival = int(arrival_delta.total_seconds())
+                
+                    if trip.has_arrived(arrival_stop,seconds_to_arrival=seconds_to_arrival,filtering_grace_period=self.FILTER_GRACE_PERIOD):
+                        continue
 
                 # Add the valid trip to the processed trips
                 filtered_trips[trip_id] = trip
@@ -498,7 +519,7 @@ class MBTABaseHandler:
                 for trip_id, trip in sorted(
                     trips.items(),
                     key=lambda item: (
-                        item[1].get_stop_by_type(sort_by).time 
+                        item[1].get_stop_by_type(sort_by).time
                         if item[1].get_stop_by_type(sort_by) and item[1].get_stop_by_type(sort_by).time is not None
                         else (today + timedelta(days=365)).astimezone()
                     )
@@ -512,7 +533,7 @@ class MBTABaseHandler:
             raise
 
     async def _update_mbta_stops_for_trips(self, trips: list[Trip]) -> None:
-        
+
         for trip in trips:
             params = {
                     'filter[route]': trip.mbta_route.id,
@@ -523,6 +544,6 @@ class MBTABaseHandler:
                 if mbta_stop.id not in self._mbta_stops_ids or not MBTAStopObjStore.get_by_id(mbta_stop.id):
                     self._mbta_trip_stops_ids.add(mbta_stop.id)
                     MBTAStopObjStore.store(mbta_stop)
-                    
+
 class MBTAStopError(Exception):
     pass
