@@ -23,7 +23,10 @@ class Trip:
     _mbta_alerts_ids: set[Optional[str]] = field(default_factory=set)
     stops: list[Optional['Stop']] = field(default_factory=list)
 
-    VEHICLE_DATA_FRESHNESS_THRESHOLD = 90  # seconds
+    VEHICLE_DATA_FRESHNESS_DATA_THRESHOLD = 60  # second, treshold to consider vehichle data fresh
+    VEHICLE_DATA_LIVENESS_DATA_THRESHOLD = 20  # seconds, treshold to consider vehichle data live and ovverride departure/arrival time for countdown
+    VEHICLE_DATA_BUFFER_TIME = 90 # seconds, how much buffer time to add to departure/arrival time when vehicle data
+    STOP_COUNTDOWN_THRESHOLD = 30 # seconds, minimum time for boarding/arriving (eg countodwn = boarding is true while COUNTDOW_TRESHOLD sec > sec to departure time)
 
     # registry
     @property
@@ -32,7 +35,6 @@ class Trip:
         mbta_route = MBTARouteObjStore.get_by_id(self._mbta_route_id)
         if mbta_route:
             return mbta_route
-        #self._mbta_route_id = None
         return None
 
     @mbta_route.setter
@@ -171,9 +173,17 @@ class Trip:
         if self.mbta_vehicle and self.mbta_vehicle.updated_at:
             now =  datetime.now().astimezone() # Ensure consistent timezone handling
             delta = (now - self.mbta_vehicle.updated_at).total_seconds()
-            return delta <= self.VEHICLE_DATA_FRESHNESS_THRESHOLD
+            return delta <= self.VEHICLE_DATA_FRESHNESS_DATA_THRESHOLD
         return False
 
+    @property
+    def is_vehicle_data_live(self) -> bool:
+        if self.mbta_vehicle and self.mbta_vehicle.updated_at:
+            now =  datetime.now().astimezone() # Ensure consistent timezone handling
+            delta = (now - self.mbta_vehicle.updated_at).total_seconds()
+            return delta <= self.VEHICLE_DATA_LIVENESS_DATA_THRESHOLD
+        return False
+    
     #departure stop
     @property
     def _departure_stop(self) -> Optional[Stop]:
@@ -207,24 +217,6 @@ class Trip:
     def departure_countdown(self) -> Optional[str]:
         return self._get_stop_countdown(StopType.DEPARTURE) if self._departure_stop else None
 
-    @property
-    def has_departed(self) -> bool:
-        stop = self._departure_stop
-        if not stop:  # No departure stop available
-            return False
-
-        now = datetime.now().astimezone()
-        seconds = (stop.departure_time.astimezone() - now).total_seconds() if stop.departure_time else stop.time
-        current_stop = self.mbta_vehicle.current_stop_sequence if self.mbta_vehicle else None
-
-        if current_stop is not None and current_stop > stop.stop_sequence:
-            return True
-        if seconds < -30:
-            return True
-
-        return False
-
-
     #arrival stop
     @property
     def _arrival_stop(self) -> Optional[Stop]:
@@ -258,24 +250,6 @@ class Trip:
     def arrival_countdown(self) -> Optional[str]:
         return self._get_stop_countdown(StopType.ARRIVAL) if self._arrival_stop else None
 
-    @property
-    def has_arrived(self) -> bool:
-        stop = self._arrival_stop
-        if not stop:  # No departure stop available
-            return False
-
-        now = datetime.now().astimezone()
-        seconds = (stop.arrival_time.astimezone() - now).total_seconds() if stop.arrival_time else stop.time
-
-        current_stop = self.mbta_vehicle.current_stop_sequence if self.mbta_vehicle else None
-
-        if current_stop is not None and current_stop > stop.stop_sequence:
-            return True
-        if seconds < -30:
-            return True
-
-        return False
-
     #alerts
     @property
     def alerts(self) -> Optional[set[str]]:
@@ -283,9 +257,8 @@ class Trip:
         if self.mbta_alerts:
             for mbta_alert in self.mbta_alerts:
                 effect = " ".join(mbta_alert.effect.split("_"))
-                lifecycle = mbta_alert.lifecycle
                 short_header = mbta_alert.short_header
-                alerts_details.add(effect + "[" + lifecycle + "]: " + short_header)
+                alerts_details.add(effect + ": " + short_header)
             return alerts_details
         return None
 
@@ -362,7 +335,6 @@ class Trip:
 
         if not stop.time:
             return None
-
         now = datetime.now().astimezone()
 
         # Calculate time deltas
@@ -372,41 +344,23 @@ class Trip:
         arrival_delta = arrival_time.astimezone() - now
         departure_delta = departure_time.astimezone() - now
 
-        seconds_arrival = int(arrival_delta.total_seconds())
-        seconds_departure = int(departure_delta.total_seconds())
+        seconds_to_arrival = int(arrival_delta.total_seconds())
+        seconds_to_departure = int(departure_delta.total_seconds())
 
-        # Vehicle data overrides
-        if self.mbta_vehicle and self.is_vehicle_data_fresh:
-
-            vehicle_stop = self.mbta_vehicle.current_stop_sequence
-            vehicle_status = self.mbta_vehicle.current_status
-
-            if vehicle_stop > stop.stop_sequence:
-                if stop_type == StopType.DEPARTURE:
-                    return "Departed"
-                if stop_type == StopType.ARRIVAL:
-                    return "Arrived"
-
-            if vehicle_stop == stop.stop_sequence:
-                if stop_type == StopType.DEPARTURE and vehicle_status == "STOPPED_AT" and 0 <= seconds_departure <= 90:
-                    return "Boarding"
-                if vehicle_status == "INCOMING_AT" and 0 <= seconds_arrival <= 90:
-                    return "Arriving"
-
-        # Handle status messages
-        if stop_type == StopType.DEPARTURE and seconds_departure < 0:
-            return "Departed"
-        if stop_type == StopType.ARRIVAL and seconds_arrival < 0:
+        if stop_type == StopType.ARRIVAL and self.has_arrived(stop=stop, seconds_to_arrival=seconds_to_arrival):
             return "Arrived"
 
-        # Prioritize boarding over arriving
-        if stop_type == StopType.DEPARTURE and 0 <= seconds_departure < 30:
+        if self.has_departed(stop=stop,seconds_to_departure=seconds_to_departure):
+            return "Departed"
+
+        if self.is_boarding(stop=stop, seconds_to_arrival=seconds_to_arrival, seconds_to_departure=seconds_to_departure):
             return "Boarding"
-        if 0 <= seconds_arrival < 30 and seconds_departure > 0:
+
+        if stop.arrival_time and self.is_arriving(stop=stop, seconds_to_arrival=seconds_to_arrival, seconds_to_departure=seconds_to_departure):
             return "Arriving"
 
         # Default to formatted time countdown
-        return self._format_time(seconds_arrival) if seconds_arrival >= 30 else None
+        return self._format_time(seconds_to_arrival) if seconds_to_arrival >= 30 else None
 
     def _get_stop_mbta_countdown(self, stop_type: StopType) -> Optional[str]:
         """Determine the countdown to a stop based on vehicle and time following
@@ -427,10 +381,10 @@ class Trip:
             if seconds < 0:
                 return None
 
-            if seconds <= 90 and self.mbta_vehicle and self.mbta_vehicle.current_status == "STOPPED_AT" and self.mbta_vehicle.current_stop_sequence == stop.stop_sequence:
+            if seconds <= self.VEHICLE_DATA_BUFFER_TIME and self.mbta_vehicle and self.mbta_vehicle.current_stop_sequence == stop.stop_sequence and  self.mbta_vehicle.current_status == "STOPPED_AT":
                 return "BRD"
-
-            if seconds <= 30:
+            
+            if seconds <= self.STOP_COUNTDOWN_THRESHOLD:
                 return "ARR"
 
             if seconds <= 60:
@@ -445,13 +399,13 @@ class Trip:
 
         return None
 
-   # Convert time to human-readable format
-    def _format_time(self, seconds: int) -> Optional[str]:
-        if seconds < 0:
+   # Convert seconds_to_arrival to human-readable format
+    def _format_time(self, seconds_to_arrival: int) -> Optional[str]:
+        if seconds_to_arrival < 0:
             return None
-        minutes = (seconds // 60) % 60
-        hours = (seconds // 3600) % 24
-        days = seconds // 86400
+        minutes = (seconds_to_arrival // 60) % 60
+        hours = (seconds_to_arrival // 3600) % 24
+        days = seconds_to_arrival // 86400
 
         if days > 0:
             return f"{days}d {hours}h {minutes}m"
@@ -460,3 +414,83 @@ class Trip:
         if minutes > 1:
             return f"{minutes} min"
         return "1 min"
+    
+    def has_departed(self, stop: Stop, seconds_to_departure: int, filtering_grace_period: Optional[int] = 0) -> bool:
+        """
+        Determines whether the transit has departed a given stop.
+        """
+        #if vehicle data (for this use case we don't need to check freshness...)
+        if self.mbta_vehicle:
+            vehicle_stop = self.mbta_vehicle.current_stop_sequence
+            # if the vehicle stop is after the departure stop 
+            if vehicle_stop and vehicle_stop > stop.stop_sequence: 
+                # if filtering grace period
+                if filtering_grace_period > 0:
+                    return seconds_to_departure + filtering_grace_period < 0
+                return True
+            
+        # If no fresh vehicle data, determine departure based on time threshold
+        return seconds_to_departure + filtering_grace_period < 0
+
+
+    def has_arrived(self, stop: Stop, seconds_to_arrival: int, filtering_grace_period: Optional[int] = 0) -> bool:
+        """
+        Determines whether the transit has arrived at a given stop.
+        """
+
+        #if vehicle data (for this use case we don't need to check freshness...)
+        if self.mbta_vehicle:
+            vehicle_stop = self.mbta_vehicle.current_stop_sequence
+            # if the vehicle is at or after the arrival stop
+            if vehicle_stop and vehicle_stop >= stop.stop_sequence:
+
+                if filtering_grace_period > 0:
+                    return seconds_to_arrival + filtering_grace_period <= 0
+                return True
+            
+        # If no fresh vehicle data, determine arrival based on time threshold
+        return seconds_to_arrival + filtering_grace_period <= 0
+
+
+    def is_boarding(self, stop: Stop, seconds_to_arrival: int, seconds_to_departure: int) -> bool:
+        """
+        Determines whether the transit is currently boarding at a given stop.
+        """
+        # If live vehicle data is available
+        if self.mbta_vehicle:
+            vehicle_stop = self.mbta_vehicle.current_stop_sequence
+            vehicle_status = self.mbta_vehicle.current_status
+
+            # Case 1: Fresh vehicle data, within the departure buffer
+            if self.is_vehicle_data_fresh:
+                if vehicle_stop == stop.stop_sequence and vehicle_status == "STOPPED_AT" and 0 <= seconds_to_departure <= self.VEHICLE_DATA_BUFFER_TIME:
+                    return True
+
+            # Case 2: Live vehicle data
+            if self.is_vehicle_data_live:
+                if vehicle_stop == stop.stop_sequence and vehicle_status == "STOPPED_AT":
+                    return True
+                else:
+                    return False  # Explicitly return False when conditions are not met
+
+        # If no vehicle data, rely strictly on schedule-based conditions
+        return (seconds_to_arrival < 0 < seconds_to_departure) or (0 <= seconds_to_departure <= self.STOP_COUNTDOWN_THRESHOLD)
+
+    def is_arriving(self, stop: Stop, seconds_to_arrival: int, seconds_to_departure: int) -> bool:
+        """
+        Determines whether the transit is currently arriving at a given stop.
+        """
+
+        # If live vehicle data is available
+        if self.mbta_vehicle and self.is_vehicle_data_live:
+            vehicle_stop = self.mbta_vehicle.current_stop_sequence
+            vehicle_status = self.mbta_vehicle.current_status
+
+            # If vehicle is approaching the stop
+            if vehicle_stop == stop.stop_sequence and vehicle_status == "INCOMING_AT":
+                return True
+            else:
+                return False
+
+        # If no vehicle data, rely strictly on schedule, arrival within the th
+        return  0 <= seconds_to_arrival <= self.STOP_COUNTDOWN_THRESHOLD and seconds_to_departure > self.STOP_COUNTDOWN_THRESHOLD
